@@ -48,22 +48,10 @@ static const char *type2string(enum connman_element_type type)
 		return "unknown";
 	case CONNMAN_ELEMENT_TYPE_ROOT:
 		return "root";
-	case CONNMAN_ELEMENT_TYPE_PROFILE:
-		return "profile";
 	case CONNMAN_ELEMENT_TYPE_DEVICE:
 		return "device";
 	case CONNMAN_ELEMENT_TYPE_NETWORK:
 		return "network";
-	case CONNMAN_ELEMENT_TYPE_SERVICE:
-		return "service";
-	case CONNMAN_ELEMENT_TYPE_IPV6:
-		return "ipv6";
-	case CONNMAN_ELEMENT_TYPE_BOOTP:
-		return "bootp";
-	case CONNMAN_ELEMENT_TYPE_ZEROCONF:
-		return "zeroconf";
-	case CONNMAN_ELEMENT_TYPE_VENDOR:
-		return "vendor";
 	}
 
 	return NULL;
@@ -635,10 +623,6 @@ void connman_element_unref(struct connman_element *element)
 			element->destruct(element);
 		free_children(element);
 		free_properties(element);
-		g_free(element->hostname);
-		g_free(element->domainname);
-		g_free(element->ipv6.address);
-		g_free(element->ipv6.network);
 		g_free(element->devname);
 		g_free(element->path);
 		g_free(element->name);
@@ -721,34 +705,7 @@ static int set_static_array_property(struct connman_element *element,
 int connman_element_get_value(struct connman_element *element,
 				enum connman_property_id id, void *value)
 {
-	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
-		return -EINVAL;
-
-	switch (id) {
-	case CONNMAN_PROPERTY_ID_HOSTNAME:
-		if (element->hostname == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		*((char **) value) = element->hostname;
-		break;
-	case CONNMAN_PROPERTY_ID_DOMAINNAME:
-		if (element->domainname == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		*((char **) value) = element->domainname;
-		break;
-	case CONNMAN_PROPERTY_ID_IPV6_GATEWAY:
-		if (element->ipv6.gateway == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		*((char **) value) = element->ipv6.gateway;
-		break;
-
-	default:
-		return -EINVAL;
-	}
-
-	return 0;
+	return -EINVAL;
 }
 
 static gboolean get_static_property(struct connman_element *element,
@@ -1203,16 +1160,6 @@ int connman_element_set_enabled(struct connman_element *element,
 
 static enum connman_service_error convert_error(enum connman_element_error error)
 {
-	switch (error) {
-	case CONNMAN_ELEMENT_ERROR_UNKNOWN:
-	case CONNMAN_ELEMENT_ERROR_FAILED:
-		break;
-	case CONNMAN_ELEMENT_ERROR_DHCP_FAILED:
-		return CONNMAN_SERVICE_ERROR_DHCP_FAILED;
-	case CONNMAN_ELEMENT_ERROR_CONNECT_FAILED:
-		return CONNMAN_SERVICE_ERROR_CONNECT_FAILED;
-	}
-
 	return CONNMAN_SERVICE_ERROR_UNKNOWN;
 }
 
