@@ -33,6 +33,23 @@ static GHashTable *session_hash;
 static connman_bool_t sessionmode;
 static struct connman_session *ecall_session;
 
+enum connman_session_trigger {
+	CONNMAN_SESSION_TRIGGER_UNKNOWN		= 0,
+	CONNMAN_SESSION_TRIGGER_SETTING		= 1,
+	CONNMAN_SESSION_TRIGGER_CONNECT		= 2,
+	CONNMAN_SESSION_TRIGGER_DISCONNECT	= 3,
+	CONNMAN_SESSION_TRIGGER_PERIODIC	= 4,
+	CONNMAN_SESSION_TRIGGER_SERVICE		= 5,
+	CONNMAN_SESSION_TRIGGER_ECALL		= 6,
+};
+
+enum connman_session_reason {
+	CONNMAN_SESSION_REASON_UNKNOWN		= 0,
+	CONNMAN_SESSION_REASON_CONNECT		= 1,
+	CONNMAN_SESSION_REASON_FREE_RIDE	= 2,
+	CONNMAN_SESSION_REASON_PERIODIC		= 3,
+};
+
 enum connman_session_roaming_policy {
 	CONNMAN_SESSION_ROAMING_POLICY_UNKNOWN		= 0,
 	CONNMAN_SESSION_ROAMING_POLICY_DEFAULT		= 1,
@@ -42,11 +59,17 @@ enum connman_session_roaming_policy {
 	CONNMAN_SESSION_ROAMING_POLICY_INTERNATIONAL	= 5,
 };
 
-struct session_info {
-	char *bearer;
+struct service_entry {
+	/* track why this service was selected */
+	enum connman_session_reason reason;
+	enum connman_service_state state;
 	const char *name;
-	char *ifname;
-	connman_bool_t connect;
+	struct connman_service *service;
+	const char *ifname;
+	const char *bearer;
+};
+
+struct session_info {
 	connman_bool_t online;
 	connman_bool_t priority;
 	GSList *allowed_bearers;
@@ -58,7 +81,7 @@ struct session_info {
 	enum connman_session_roaming_policy roaming_policy;
 	unsigned int marker;
 
-	struct connman_service *service;
+	struct service_entry *entry;
 };
 
 struct connman_session {
@@ -69,10 +92,11 @@ struct connman_session {
 
 	connman_bool_t append_all;
 	connman_bool_t info_dirty;
-	struct session_info info;
-	struct session_info info_last;
+	struct session_info *info;
+	struct session_info *info_last;
 
 	GSequence *service_list;
+	GHashTable *service_hash;
 };
 
 struct bearer_info {
@@ -81,11 +105,49 @@ struct bearer_info {
 	enum connman_service_type service_type;
 };
 
+static const char *trigger2string(enum connman_session_trigger trigger)
+{
+	switch (trigger) {
+	case CONNMAN_SESSION_TRIGGER_UNKNOWN:
+		break;
+	case CONNMAN_SESSION_TRIGGER_SETTING:
+		return "setting";
+	case CONNMAN_SESSION_TRIGGER_CONNECT:
+		return "connect";
+	case CONNMAN_SESSION_TRIGGER_DISCONNECT:
+		return "disconnect";
+	case CONNMAN_SESSION_TRIGGER_PERIODIC:
+		return "periodic";
+	case CONNMAN_SESSION_TRIGGER_SERVICE:
+		return "service";
+	case CONNMAN_SESSION_TRIGGER_ECALL:
+		return "ecall";
+	}
+
+	return NULL;
+}
+
+static const char *reason2string(enum connman_session_reason reason)
+{
+	switch (reason) {
+	case CONNMAN_SESSION_REASON_UNKNOWN:
+		break;
+	case CONNMAN_SESSION_REASON_CONNECT:
+		return "connect";
+	case CONNMAN_SESSION_REASON_FREE_RIDE:
+		return "free-ride";
+	case CONNMAN_SESSION_REASON_PERIODIC:
+		return "periodic";
+	}
+
+	return NULL;
+}
+
 static const char *roamingpolicy2string(enum connman_session_roaming_policy policy)
 {
 	switch (policy) {
 	case CONNMAN_SESSION_ROAMING_POLICY_UNKNOWN:
-		break;
+		return "unknown";
 	case CONNMAN_SESSION_ROAMING_POLICY_DEFAULT:
 		return "default";
 	case CONNMAN_SESSION_ROAMING_POLICY_ALWAYS:
@@ -160,36 +222,6 @@ static char *service2bearer(enum connman_service_type type)
 	return "";
 }
 
-static char *session2bearer(struct connman_session *session)
-{
-	struct session_info *info = &session->info;
-	GSList *list;
-	struct bearer_info *bearer_info;
-	enum connman_service_type type;
-
-	if (info->service == NULL)
-		return "";
-
-	type = connman_service_get_type(info->service);
-
-	for (list = info->allowed_bearers;
-			list != NULL; list = list->next) {
-		bearer_info = list->data;
-
-		if (bearer_info->match_all)
-			return service2bearer(type);
-
-		if (bearer_info->service_type == CONNMAN_SERVICE_TYPE_UNKNOWN)
-			return bearer_info->name;
-
-		if (bearer_info->service_type == type)
-			return service2bearer(type);
-	}
-
-	return "";
-
-}
-
 static void cleanup_bearer_info(gpointer data, gpointer user_data)
 {
 	struct bearer_info *info = data;
@@ -306,17 +338,11 @@ static void append_ipconfig_ipv6(DBusMessageIter *iter, void *user_data)
 static void append_notify(DBusMessageIter *dict,
 					struct connman_session *session)
 {
-	struct session_info *info = &session->info;
-	struct session_info *info_last = &session->info_last;
+	struct session_info *info = session->info;
+	struct session_info *info_last = session->info_last;
 	const char *policy;
-
-	if (session->append_all == TRUE ||
-			info->bearer != info_last->bearer) {
-		connman_dbus_dict_append_basic(dict, "Bearer",
-						DBUS_TYPE_STRING,
-						&info->bearer);
-		info_last->bearer = info->bearer;
-	}
+	struct connman_service *service;
+	const char *name, *ifname, *bearer;
 
 	if (session->append_all == TRUE ||
 			info->online != info_last->online) {
@@ -327,28 +353,40 @@ static void append_notify(DBusMessageIter *dict,
 	}
 
 	if (session->append_all == TRUE ||
-			info->name != info_last->name) {
+			info->entry != info_last->entry) {
+		if (info->entry == NULL) {
+			name = "";
+			ifname = "";
+			service = NULL;
+			bearer = "";
+		} else {
+			name = info->entry->name;
+			ifname = info->entry->ifname;
+			service = info->entry->service;
+			bearer = info->entry->bearer;
+		}
+
 		connman_dbus_dict_append_basic(dict, "Name",
 						DBUS_TYPE_STRING,
-						&info->name);
-		info_last->name = info->name;
-	}
+						&name);
 
-	if (session->append_all == TRUE ||
-			info->service != info_last->service) {
 		connman_dbus_dict_append_dict(dict, "IPv4",
 						append_ipconfig_ipv4,
-						info->service);
+						service);
 
 		connman_dbus_dict_append_dict(dict, "IPv6",
 						append_ipconfig_ipv6,
-						info->service);
+						service);
 
 		connman_dbus_dict_append_basic(dict, "Interface",
 						DBUS_TYPE_STRING,
-						&info->ifname);
+						&ifname);
 
-		info_last->service = info->service;
+		connman_dbus_dict_append_basic(dict, "Bearer",
+						DBUS_TYPE_STRING,
+						&bearer);
+
+		info_last->entry = info->entry;
 	}
 
 
@@ -430,8 +468,9 @@ static void append_notify(DBusMessageIter *dict,
 	session->info_dirty = FALSE;
 }
 
-static int session_notify(struct connman_session *session)
+static gboolean session_notify(gpointer user_data)
 {
+	struct connman_session *session = user_data;
 	DBusMessage *msg;
 	DBusMessageIter array, dict;
 
@@ -445,7 +484,7 @@ static int session_notify(struct connman_session *session)
 						CONNMAN_NOTIFICATION_INTERFACE,
 						"Update");
 	if (msg == NULL)
-		return -ENOMEM;
+		return FALSE;
 
 	dbus_message_iter_init_append(msg, &array);
 	connman_dbus_dict_open(&array, &dict);
@@ -458,31 +497,31 @@ static int session_notify(struct connman_session *session)
 
 	session->info_dirty = FALSE;
 
-	return 0;
+	return FALSE;
 }
 
 static void ipconfig_ipv4_changed(struct connman_session *session)
 {
-	struct session_info *info = &session->info;
+	struct session_info *info = session->info;
 
 	connman_dbus_setting_changed_dict(session->owner, session->notify_path,
 						"IPv4", append_ipconfig_ipv4,
-						info->service);
+						info->entry->service);
 }
 
 static void ipconfig_ipv6_changed(struct connman_session *session)
 {
-	struct session_info *info = &session->info;
+	struct session_info *info = session->info;
 
 	connman_dbus_setting_changed_dict(session->owner, session->notify_path,
 						"IPv6", append_ipconfig_ipv6,
-						info->service);
+						info->entry->service);
 }
 
 static connman_bool_t service_type_match(struct connman_session *session,
 					struct connman_service *service)
 {
-	struct session_info *info = &session->info;
+	struct session_info *info = session->info;
 	GSList *list;
 
 	for (list = info->allowed_bearers;
@@ -548,7 +587,7 @@ static gint sort_allowed_bearers(struct connman_service *service_a,
 					struct connman_service *service_b,
 					struct connman_session *session)
 {
-	struct session_info *info = &session->info;
+	struct session_info *info = session->info;
 	GSList *list;
 	enum connman_service_type type_a, type_b;
 	int weight_a, weight_b;
@@ -606,18 +645,26 @@ static gint sort_services(gconstpointer a, gconstpointer b, gpointer user_data)
 static void cleanup_session(gpointer user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = &session->info;
+	struct session_info *info = session->info;
 
 	DBG("remove %s", session->session_path);
 
+	g_hash_table_destroy(session->service_hash);
 	g_sequence_free(session->service_list);
 
+	if (info->entry != NULL &&
+			info->entry->reason == CONNMAN_SESSION_REASON_CONNECT) {
+		__connman_service_disconnect(info->entry->service);
+	}
+
 	g_slist_foreach(info->allowed_bearers, cleanup_bearer_info, NULL);
 	g_slist_free(info->allowed_bearers);
 
 	g_free(session->owner);
 	g_free(session->session_path);
 	g_free(session->notify_path);
+	g_free(session->info);
+	g_free(session->info_last);
 
 	g_free(session);
 }
@@ -681,42 +728,132 @@ static DBusMessage *destroy_session(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static gboolean session_changed_connect(gpointer user_data)
+static connman_bool_t is_connected(enum connman_service_state state)
 {
-	struct connman_session *session = user_data;
-	struct session_info *info = &session->info;
-
-	__connman_service_connect(info->service);
+	switch (state) {
+	case CONNMAN_SERVICE_STATE_UNKNOWN:
+	case CONNMAN_SERVICE_STATE_IDLE:
+	case CONNMAN_SERVICE_STATE_ASSOCIATION:
+	case CONNMAN_SERVICE_STATE_CONFIGURATION:
+	case CONNMAN_SERVICE_STATE_DISCONNECT:
+	case CONNMAN_SERVICE_STATE_FAILURE:
+		break;
+	case CONNMAN_SERVICE_STATE_READY:
+	case CONNMAN_SERVICE_STATE_ONLINE:
+		return TRUE;
+	}
 
 	return FALSE;
 }
 
-static void update_service(struct connman_session *session)
+static connman_bool_t explicit_connect(enum connman_session_reason reason)
 {
-	struct session_info *info = &session->info;
-	int idx;
+	switch (reason) {
+	case CONNMAN_SESSION_REASON_UNKNOWN:
+	case CONNMAN_SESSION_REASON_FREE_RIDE:
+		break;
+	case CONNMAN_SESSION_REASON_CONNECT:
+	case CONNMAN_SESSION_REASON_PERIODIC:
+		return TRUE;
+	}
 
-	if (info->service != NULL) {
-		info->bearer = session2bearer(session);
-		info->online = __connman_service_is_connected(info->service);
-		info->name = __connman_service_get_name(info->service);
-		idx = __connman_service_get_index(info->service);
-		info->ifname = connman_inet_ifname(idx);
+	return FALSE;
+}
 
-	} else {
-		info->bearer = "";
-		info->online = FALSE;
-		info->name = "";
-		info->ifname = "";
+static void test_and_disconnect(struct connman_session *session)
+{
+	struct session_info *info = session->info;
+
+	if (info->entry == NULL)
+		return;
+
+	DBG("session %p reason %s service %p state %d",
+		session, reason2string(info->entry->reason),
+		info->entry->service, info->entry->state);
+
+	if (explicit_connect(info->entry->reason) == FALSE)
+		goto out;
+
+	if (__connman_service_session_dec(info->entry->service) == TRUE)
+		goto out;
+
+	if (ecall_session != NULL && ecall_session != session)
+		goto out;
+
+	__connman_service_disconnect(info->entry->service);
+
+	/*
+	 * TODO: We should mark this entry as pending work. In case
+	 * disconnect fails we just unassign this session from the
+	 * service and can't do anything later on it
+	 */
+
+out:
+	info->online = FALSE;
+	info->entry = NULL;
+}
+
+static void select_and_connect(struct connman_session *session,
+				enum connman_session_reason reason)
+{
+	struct session_info *info = session->info;
+	struct service_entry *entry = NULL;
+	GSequenceIter *iter;
+	connman_bool_t do_connect = FALSE;
+
+	DBG("session %p reason %s", session, reason2string(reason));
+
+	iter = g_sequence_get_begin_iter(session->service_list);
+
+	while (g_sequence_iter_is_end(iter) == FALSE) {
+		entry = g_sequence_get(iter);
+
+		switch (entry->state) {
+		case CONNMAN_SERVICE_STATE_ASSOCIATION:
+		case CONNMAN_SERVICE_STATE_CONFIGURATION:
+		case CONNMAN_SERVICE_STATE_READY:
+		case CONNMAN_SERVICE_STATE_ONLINE:
+			/* connecting or connected */
+			break;
+		case CONNMAN_SERVICE_STATE_IDLE:
+		case CONNMAN_SERVICE_STATE_DISCONNECT:
+			if (explicit_connect(reason) == TRUE)
+				do_connect = TRUE;
+			break;
+		case CONNMAN_SERVICE_STATE_UNKNOWN:
+		case CONNMAN_SERVICE_STATE_FAILURE:
+			entry = NULL;
+			break;
+		}
+
+		if (entry != NULL)
+			break;
+
+		iter = g_sequence_iter_next(iter);
+	}
+
+	if (info->entry != NULL && info->entry != entry)
+		test_and_disconnect(session);
+
+	if (entry != NULL) {
+		info->entry = entry;
+		info->entry->reason = reason;
+
+		if (explicit_connect(reason) == TRUE)
+			__connman_service_session_inc(info->entry->service);
+
+		if (do_connect == TRUE)
+			__connman_service_connect(info->entry->service);
+		else
+			info->online = is_connected(entry->state);
 	}
 }
 
-static void session_changed(struct connman_session *session)
+static void session_changed(struct connman_session *session,
+				enum connman_session_trigger trigger)
 {
-	struct session_info *info = &session->info;
-	struct session_info *info_last = &session->info_last;
-	struct connman_service *service = NULL;
-	GSourceFunc callback = NULL;
+	struct session_info *info = session->info;
+	struct session_info *info_last = session->info_last;
 	GSequenceIter *iter;
 
 	/*
@@ -724,76 +861,97 @@ static void session_changed(struct connman_session *session)
 	 * play a bit around. So we are going to improve it step by step.
 	 */
 
-	if (info->ecall == TRUE && session != ecall_session)
-		goto out;
+	DBG("session %p trigger %s", session, trigger2string(trigger));
 
-	if (info->connect == FALSE) {
-		if (info->service != NULL)
-			__connman_service_disconnect(info->service);
-		info->service = NULL;
-		goto out;
-	}
+	switch (trigger) {
+	case CONNMAN_SESSION_TRIGGER_UNKNOWN:
+		DBG("ignore session changed event");
+		return;
+	case CONNMAN_SESSION_TRIGGER_SETTING:
+		if (info->entry != NULL) {
+			iter = g_hash_table_lookup(session->service_hash,
+							info->entry->service);
+			if (iter == NULL) {
+				/*
+				 * This service is not part of this
+				 * session anymore.
+				 */
+				test_and_disconnect(session);
+			}
+		}
 
-	iter = g_sequence_get_begin_iter(session->service_list);
+		if (info->online == FALSE) {
+			select_and_connect(session,
+					CONNMAN_SESSION_REASON_FREE_RIDE);
+		}
 
-	while (g_sequence_iter_is_end(iter) == FALSE) {
-		service = g_sequence_get(iter);
-
-		if (__connman_service_is_connecting(service) == TRUE ||
-				__connman_service_is_connected(service) == TRUE) {
+		break;
+	case CONNMAN_SESSION_TRIGGER_CONNECT:
+		if (info->online == TRUE) {
+			info->entry->reason = CONNMAN_SESSION_REASON_CONNECT;
+			__connman_service_session_inc(info->entry->service);
 			break;
 		}
 
-		if (__connman_service_is_idle(service) == TRUE &&
-				info->connect == TRUE) {
-			callback = session_changed_connect;
+		select_and_connect(session,
+				CONNMAN_SESSION_REASON_CONNECT);
+
+		break;
+	case CONNMAN_SESSION_TRIGGER_DISCONNECT:
+		test_and_disconnect(session);
+
+		break;
+	case CONNMAN_SESSION_TRIGGER_PERIODIC:
+		if (info->online == TRUE) {
+			info->entry->reason = CONNMAN_SESSION_REASON_PERIODIC;
+			__connman_service_session_inc(info->entry->service);
 			break;
 		}
 
-		service = NULL;
+		select_and_connect(session,
+				CONNMAN_SESSION_REASON_PERIODIC);
 
-		iter = g_sequence_iter_next(iter);
+		break;
+	case CONNMAN_SESSION_TRIGGER_SERVICE:
+		if (info->online == TRUE)
+			break;
+
+		if (info->stay_connected == TRUE) {
+			DBG("StayConnected");
+			select_and_connect(session,
+					CONNMAN_SESSION_REASON_CONNECT);
+
+			break;
+		}
+
+		select_and_connect(session,
+				CONNMAN_SESSION_REASON_FREE_RIDE);
+
+		break;
+	case CONNMAN_SESSION_TRIGGER_ECALL:
+		if (info->online == FALSE && info->entry->service != NULL)
+			test_and_disconnect(session);
+
+		break;
 	}
 
-	if (info->service != NULL && info->service != service) {
-		__connman_service_disconnect(info->service);
-		info->service = NULL;
-	}
+	if (info->entry != info_last->entry)
+		session->info_dirty = TRUE;
 
-	if (service != NULL) {
-		info->service = service;
-
-		if (callback != NULL)
-			callback(session);
-	}
-
-out:
-	if (info->service != info_last->service)
-		update_service(session);
-}
-
-static gboolean session_cb(gpointer user_data)
-{
-	struct connman_session *session = user_data;
-
-	session_changed(session);
 	session_notify(session);
-
-	return FALSE;
 }
 
 static DBusMessage *connect_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = &session->info;
 
 	DBG("session %p", session);
 
-	info->connect = TRUE;
-	session->info_dirty = TRUE;
+	if (ecall_session != NULL && ecall_session != session)
+		return __connman_error_failed(msg, EBUSY);
 
-	g_timeout_add_seconds(0, session_cb, session);
+	session_changed(session, CONNMAN_SESSION_TRIGGER_CONNECT);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -802,43 +960,90 @@ static DBusMessage *disconnect_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = &session->info;
 
 	DBG("session %p", session);
 
-	info->connect = FALSE;
-	session->info_dirty = TRUE;
+	if (ecall_session != NULL && ecall_session != session)
+		return __connman_error_failed(msg, EBUSY);
 
-	g_timeout_add_seconds(0, session_cb, session);
+	session_changed(session, CONNMAN_SESSION_TRIGGER_DISCONNECT);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static void print_name(gpointer data, gpointer user_data)
+static struct service_entry *create_service_entry(struct connman_service *service,
+					const char *name,
+					enum connman_service_state state)
 {
-	struct connman_service *service = data;
+	struct service_entry *entry;
+	enum connman_service_type type;
+	int idx;
 
-	DBG("service %p type %s name %s", service,
-		service2bearer(connman_service_get_type(service)),
-		__connman_service_get_name(service));
+	entry = g_try_new0(struct service_entry, 1);
+	if (entry == NULL)
+		return entry;
+
+	entry->reason = CONNMAN_SESSION_REASON_UNKNOWN;
+	entry->state = state;
+	entry->name = name;
+	entry->service = service;
+
+	idx = __connman_service_get_index(entry->service);
+	entry->ifname = connman_inet_ifname(idx);
+	if (entry->ifname == NULL)
+		entry->ifname = "";
+
+	type = connman_service_get_type(entry->service);
+	entry->bearer = service2bearer(type);
+
+	return entry;
+}
+
+static void destroy_service_entry(gpointer data)
+{
+	struct service_entry *entry = data;
+
+	g_free(entry);
 }
 
 static void update_allowed_bearers(struct connman_session *session)
 {
-	if (session->service_list != NULL)
+	struct service_entry *entry;
+	GSequenceIter *iter;
+
+	if (session->service_list != NULL) {
+		g_hash_table_remove_all(session->service_hash);
 		g_sequence_free(session->service_list);
+	}
 
 	session->service_list = __connman_service_get_list(session,
-								service_match);
+							service_match,
+							create_service_entry,
+							destroy_service_entry);
+
 	g_sequence_sort(session->service_list, sort_services, session);
-	g_sequence_foreach(session->service_list, print_name, NULL);
+
+	iter = g_sequence_get_begin_iter(session->service_list);
+
+	while (g_sequence_iter_is_end(iter) == FALSE) {
+		entry = g_sequence_get(iter);
+
+		DBG("service %p type %s name %s", entry->service,
+			service2bearer(connman_service_get_type(entry->service)),
+			entry->name);
+
+		g_hash_table_replace(session->service_hash,
+					entry->service, iter);
+
+		iter = g_sequence_iter_next(iter);
+	}
 
 	session->info_dirty = TRUE;
 }
 
 static void update_ecall_sessions(struct connman_session *session)
 {
-	struct session_info *info = &session->info;
+	struct session_info *info = session->info;
 	struct connman_session *session_iter;
 	GHashTableIter iter;
 	gpointer key, value;
@@ -851,20 +1056,20 @@ static void update_ecall_sessions(struct connman_session *session)
 		if (session_iter == session)
 			continue;
 
-		session_iter->info.ecall = info->ecall;
+		session_iter->info->ecall = info->ecall;
 		session_iter->info_dirty = TRUE;
 
-		g_timeout_add_seconds(0, session_cb, session_iter);
+		session_changed(session_iter, CONNMAN_SESSION_TRIGGER_ECALL);
 	}
 }
 
 static void update_ecall(struct connman_session *session)
 {
-	struct session_info *info = &session->info;
-	struct session_info *info_last = &session->info_last;
+	struct session_info *info = session->info;
+	struct session_info *info_last = session->info_last;
 
-	DBG("ecall_session %p ecall %d -> %d", ecall_session,
-		info_last->ecall, info->ecall);
+	DBG("session %p ecall_session %p ecall %d -> %d", session,
+		ecall_session, info_last->ecall, info->ecall);
 
 	if (ecall_session == NULL) {
 		if (!(info_last->ecall == FALSE && info->ecall == TRUE))
@@ -894,8 +1099,8 @@ static DBusMessage *change_session(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct session_info *info = &session->info;
-	struct session_info *info_last = &session->info_last;
+	struct session_info *info = session->info;
+	struct session_info *info_last = session->info_last;
 	DBusMessageIter iter, value;
 	const char *name;
 	GSList *allowed_bearers;
@@ -944,7 +1149,7 @@ static DBusMessage *change_session(DBusConnection *conn,
 
 			if (info_last->avoid_handover != info->avoid_handover)
 				session->info_dirty = TRUE;
-			} else if (g_str_equal(name, "StayConnected") == TRUE) {
+		} else if (g_str_equal(name, "StayConnected") == TRUE) {
 			dbus_message_iter_get_basic(&value,
 					&info->stay_connected);
 
@@ -994,7 +1199,7 @@ static DBusMessage *change_session(DBusConnection *conn,
 	}
 
 	if (session->info_dirty == TRUE)
-		session_cb(session);
+		session_changed(session, CONNMAN_SESSION_TRIGGER_SETTING);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 
@@ -1015,7 +1220,7 @@ int __connman_session_create(DBusMessage *msg)
 	const char *owner, *notify_path;
 	char *session_path = NULL;
 	DBusMessageIter iter, array;
-	struct connman_session *session;
+	struct connman_session *session = NULL;
 	struct session_info *info, *info_last;
 
 	connman_bool_t priority = FALSE, avoid_handover = FALSE;
@@ -1118,6 +1323,7 @@ int __connman_session_create(DBusMessage *msg)
 
 	session = g_hash_table_lookup(session_hash, session_path);
 	if (session != NULL) {
+		session = NULL;
 		err = -EEXIST;
 		goto err;
 	}
@@ -1128,8 +1334,20 @@ int __connman_session_create(DBusMessage *msg)
 		goto err;
 	}
 
-	info = &session->info;
-	info_last = &session->info_last;
+	session->info = g_try_new0(struct session_info, 1);
+	if (session->info == NULL) {
+		err = -ENOMEM;
+		goto err;
+	}
+
+	session->info_last = g_try_new0(struct session_info, 1);
+	if (session->info_last == NULL) {
+		err = -ENOMEM;
+		goto err;
+	}
+
+	info = session->info;
+	info_last = session->info_last;
 
 	session->owner = g_strdup(owner);
 	session->session_path = session_path;
@@ -1138,7 +1356,9 @@ int __connman_session_create(DBusMessage *msg)
 		g_dbus_add_disconnect_watch(connection, session->owner,
 					owner_disconnect, session, NULL);
 
-	info->bearer = "";
+	session->service_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
+						NULL, NULL);
+
 	info->online = FALSE;
 	info->priority = priority;
 	info->avoid_handover = avoid_handover;
@@ -1147,7 +1367,7 @@ int __connman_session_create(DBusMessage *msg)
 	info->idle_timeout = idle_timeout;
 	info->ecall = ecall;
 	info->roaming_policy = roaming_policy;
-	info->service = NULL;
+	info->entry = NULL;
 	info->marker = 0;
 
 	if (allowed_bearers == NULL) {
@@ -1184,33 +1404,41 @@ int __connman_session_create(DBusMessage *msg)
 
 
 	update_allowed_bearers(session);
-	update_service(session);
 	if (info->ecall == TRUE) {
 		ecall_session = session;
 		update_ecall_sessions(session);
 	}
 
-	info_last->bearer = info->bearer;
-	info_last->online = info->priority;
+	info_last->online = info->online;
+	info_last->priority = info->priority;
 	info_last->avoid_handover = info->avoid_handover;
 	info_last->stay_connected = info->stay_connected;
 	info_last->periodic_connect = info->periodic_connect;
 	info_last->idle_timeout = info->idle_timeout;
 	info_last->ecall = info->ecall;
 	info_last->roaming_policy = info->roaming_policy;
-	info_last->service = info->service;
+	info_last->entry = info->entry;
 	info_last->marker = info->marker;
 	info_last->allowed_bearers = info->allowed_bearers;
 
 	session->info_dirty = TRUE;
 	session->append_all = TRUE;
 
-	g_timeout_add_seconds(0, session_cb, session);
+	session_changed(session, CONNMAN_SESSION_TRIGGER_SETTING);
 
 	return 0;
 
 err:
 	connman_error("Failed to create session");
+
+	if (session != NULL) {
+		if (session->info_last != NULL)
+			g_free(session->info_last);
+		if (session->info != NULL)
+			g_free(session->info);
+		g_free(session);
+	}
+
 	g_free(session_path);
 
 	g_slist_foreach(allowed_bearers, cleanup_bearer_info, NULL);
@@ -1266,6 +1494,7 @@ void __connman_session_set_mode(connman_bool_t enable)
 static void service_add(struct connman_service *service)
 {
 	GHashTableIter iter;
+	GSequenceIter *iter_service_list;
 	gpointer key, value;
 	struct connman_session *session;
 
@@ -1279,56 +1508,43 @@ static void service_add(struct connman_service *service)
 		if (service_match(session, service) == FALSE)
 			continue;
 
-		g_sequence_insert_sorted(session->service_list, service,
-						sort_services, session);
+		iter_service_list =
+			g_sequence_insert_sorted(session->service_list,
+							service, sort_services,
+							session);
 
-		session_cb(session);
+		g_hash_table_replace(session->service_hash, service,
+					iter_service_list);
+
+		session_changed(session, CONNMAN_SESSION_TRIGGER_SERVICE);
 	}
 }
 
-static gint service_in_session(gconstpointer a, gconstpointer b,
-				gpointer user_data)
-{
-	if (a == b)
-		return 0;
-
-	return -1;
-}
-
 static void service_remove(struct connman_service *service)
 {
 
 	GHashTableIter iter;
 	gpointer key, value;
-	GSequenceIter *seq_iter;
 	struct connman_session *session;
 	struct session_info *info;
-	struct connman_service *found_service;
 
 	DBG("service %p", service);
 
 	g_hash_table_iter_init(&iter, session_hash);
 
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
+		GSequenceIter *iter;
 		session = value;
-		info = &session->info;
+		info = session->info;
 
-		if (session->service_list == NULL)
+		iter = g_hash_table_lookup(session->service_hash, service);
+		if (iter == NULL)
 			continue;
 
-		seq_iter = g_sequence_search(session->service_list, service,
-						service_in_session, NULL);
-		if (g_sequence_iter_is_end(seq_iter) == TRUE)
-			continue;
+		g_sequence_remove(iter);
 
-		g_sequence_remove(seq_iter);
-
-		found_service = g_sequence_get(seq_iter);
-		if (found_service != info->service)
-			continue;
-
-		info->service = NULL;
-		session_cb(session);
+		info->entry = NULL;
+		session_changed(session, CONNMAN_SESSION_TRIGGER_SERVICE);
 	}
 }
 
@@ -1338,8 +1554,7 @@ static void service_state_changed(struct connman_service *service,
 	GHashTableIter iter;
 	gpointer key, value;
 	struct connman_session *session;
-	struct session_info *info;
-	connman_bool_t online;
+	struct session_info *info, *info_last;
 
 	DBG("service %p state %d", service, state);
 
@@ -1347,16 +1562,18 @@ static void service_state_changed(struct connman_service *service,
 
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
 		session = value;
-		info = &session->info;
+		info = session->info;
+		info_last = session->info_last;
 
-		if (info->service == service) {
-			online = __connman_service_is_connected(service);
-			if (info->online == online)
-				continue;
-
-			info->online = online;
-			session_cb(session);
+		if (info->entry != NULL && info->entry->service == service) {
+			info->entry->state = state;
+			info->online = is_connected(info->entry->state);
+			if (info_last->online != info->online)
+				session->info_dirty = TRUE;
 		}
+
+		session_changed(session,
+				CONNMAN_SESSION_TRIGGER_SERVICE);
 	}
 }
 
@@ -1377,9 +1594,9 @@ static void ipconfig_changed(struct connman_service *service,
 
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
 		session = value;
-		info = &session->info;
+		info = session->info;
 
-		if (info->service == service) {
+		if (info->entry != NULL && info->entry->service == service) {
 			if (type == CONNMAN_IPCONFIG_TYPE_IPV4)
 				ipconfig_ipv4_changed(session);
 			else if (type == CONNMAN_IPCONFIG_TYPE_IPV6)
